package com.starcharge.wios.auth.bean;

import java.nio.charset.StandardCharsets;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.HmacAlgorithms;
import org.apache.commons.codec.digest.HmacUtils;

import com.alibaba.fastjson.JSONObject;

/**
 * JWT解析类：jwt数据结构为Header.Payload.Signature，其中Header和Payload是Base64UrlEncode算法生成的字符串，明文是json字符串
 * 
 * @author lilin
 * @version [版本号, 2019年5月5日]
 */
public class JwtToken {
    /**
     * 未认证
     */
    public static final String REQUEST_UNAUTHORIZED = "jwt_401";
    
    /**
     * ip认证失败
     */
    public static final String REQUEST_IP_CHANGED = "jwt_402";
    
    /**
     * 是否是合法的jwtToken
     */
    private boolean validate;
    
    /**
     * 异常原因
     */
    private String errMsg;
    
    /**
     * 头部json
     */
    private JSONObject header;
    
    /**
     * body部json
     */
    private JSONObject payload;
    
    /**
     * header + "." + payload的密文部分
     */
    private String encryptionStr;
    
    /**
     * 签名
     */
    private String signature;
    
    private JwtToken() {
        validate = false;
    }
    
    /**
     * 根据String类型的token获取jwtToken对象
     *
     * @param token token
     * @return jwtToken
     */
    public static JwtToken getJwtToken(String token) {
        JwtToken jwtToken = new JwtToken();
        if (token == null) {
            jwtToken.setErrMsg("token为空");
            return jwtToken;
        }
        String[] parts = token.split("\\.");
        if (parts.length != 3) {
            jwtToken.setErrMsg("token信息不全");
            return jwtToken;
        }
        jwtToken.setEncryptionStr(parts[0] + '.' + parts[1]);
        // 考虑不做多余性能开销，header暂时不解析
        try {
            jwtToken.setPayload(JSONObject.parseObject(new String(Base64.decodeBase64(parts[1]), StandardCharsets.UTF_8)));
        } catch (Exception e) {
            jwtToken.setErrMsg("Payload不合法");
            return jwtToken;
        }
        jwtToken.setSignature(parts[2]);
        jwtToken.setValidate(true);
        return jwtToken;
    }
    
    /**
     * 创建jwtToken
     *
     * @param header header
     * @param payload payload
     * @param secret 秘钥
     * @return jwtToken字符串
     */
    public static String createJwtToken(String header, String payload, String secret) {
        StringBuilder encryStr = new StringBuilder();
        encryStr.append(Base64.encodeBase64String(header.getBytes(StandardCharsets.UTF_8)));
        encryStr.append('.');
        encryStr.append(Base64.encodeBase64String(payload.getBytes(StandardCharsets.UTF_8)));
        String signature = new HmacUtils(HmacAlgorithms.HMAC_SHA_256, secret).hmacHex(encryStr.toString());
        
        encryStr.append('.');
        encryStr.append(signature);
        return encryStr.toString();
    }
    
    /**
     * 校验jwtToken是否合法
     *
     * @param secret 秘钥
     * @return true/false
     */
    public boolean validate(String secret) {
        if (!isValidate()) {
            return false;
        }
        // 默认使用HMAC sha256算法签名
        return signature.equals(new HmacUtils(HmacAlgorithms.HMAC_SHA_256, secret).hmacHex(encryptionStr));
    }
    
    public boolean isValidate() {
        return validate;
    }
    
    public void setValidate(boolean validate) {
        this.validate = validate;
    }
    
    public String getErrMsg() {
        return errMsg;
    }
    
    public void setErrMsg(String errMsg) {
        this.errMsg = errMsg;
    }
    
    public JSONObject getHeader() {
        return header;
    }
    
    public void setHeader(JSONObject header) {
        this.header = header;
    }
    
    public JSONObject getPayload() {
        return payload;
    }
    
    public void setPayload(JSONObject payload) {
        this.payload = payload;
    }
    
    public String getSignature() {
        return signature;
    }
    
    public void setSignature(String signature) {
        this.signature = signature;
    }
    
    public String getEncryptionStr() {
        return encryptionStr;
    }
    
    public void setEncryptionStr(String encryptionStr) {
        this.encryptionStr = encryptionStr;
    }
}
