package com.starcharge.wios.auth.controller;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.auth.bo.RoleBo;
import com.starcharge.wios.auth.dao.model.Role;
import com.starcharge.wios.auth.service.RoleService;
import com.starcharge.wios.auth.vo.AbsractTreeNode;
import com.starcharge.wios.auth.vo.MenuTreeVo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 角色相关api
 * 
 * @author lilin
 * @version [版本号, 2020年5月20日]
 */
@RestController
@RequestMapping("api")
@Api(tags = "角色管理接口")
public class RoleController {
    /**
     * 角色service
     */
    @Resource
    private RoleService roleService;
    
    /**
     * 查询角色列表
     *
     * @param query 查询条件
     * @return 角色列表
     */
    @GetMapping(value = "role/list")
    @ApiOperation(value = "查询角色列表")
    public BaseResponse<List<Role>> queryList(RoleBo role) {
        return new BaseResponse<List<Role>>(roleService.queryList(role));
    }
    
    /**
     * 新增角色
     *
     * @param role 角色信息
     * @return 新增成功的数量
     */
    @PostMapping(value = "role/add")
    @ApiOperation(value = "添加角色")
    public BaseResponse<Integer> addRole(RoleBo role) {
        // if (role.getParentId() == null) {
        // throw new ServiceException("parentId不能为空");
        // }
        if (!StringUtils.isEmpty(role.getName()) && role.getName().length() > 32) {
            throw new ServiceException("角色名称过长");
        }
        return new BaseResponse<Integer>(roleService.addRole(role));
    }
    
    /**
     * 修改角色
     *
     * @param role 角色信息
     * @return 修改成功的数量
     */
    @PostMapping(value = "role/update")
    @ApiOperation(value = "修改角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键", dataType = "int", paramType = "query", required = true),})
    public BaseResponse<Integer> updateRole(RoleBo role) {
        if (role.getId() == null) {
            throw new ServiceException("缺少参数");
        }
        return new BaseResponse<Integer>(roleService.updateRole(role));
    }
    
    /**
     * 删除角色
     *
     * @param id 角色id
     * @return 删除成功的数量
     */
    @PostMapping(value = "role/delete")
    @ApiOperation(value = "删除角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "主键", dataType = "int", paramType = "query", required = true),})
    public BaseResponse<Integer> deleteRole(Integer id) {
        return new BaseResponse<Integer>(roleService.deleteRole(id));
    }
    
    /**
     * 查询角色详细信息
     *
     * @param id 角色id
     * @return 角色详细信息
     */
    // @OpenAPIMethod(methodName = "role/get")
    // public RoleDetailVo getRole(Integer id) {
    // return roleService.getRoleDetail(id);
    // }
    
    /**
     * 查询角色菜单权限
     *
     * @param roleId 角色id
     * @return 角色菜单权限
     */
    @GetMapping(value = "role/menuList")
    @ApiOperation(value = "查询角色菜单权限")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleId", value = "角色id", dataType = "int", paramType = "query", required = true),
    })
    public BaseResponse<List<MenuTreeVo>> getRoleMenuList(Integer roleId) {
        return new BaseResponse<List<MenuTreeVo>>(roleService.getRoleMenuList(roleId));
    }
    
    /**
     * 获取左侧菜单列表
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "menuList")
    @ApiOperation(value = "获取左侧菜单列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "accountId", value = "账号id", dataType = "int", paramType = "query", required = true),})
    public BaseResponse<List<AbsractTreeNode>> getMenuList(Integer accountId) {
        return new BaseResponse<List<AbsractTreeNode>>(roleService.getMenuList(accountId));
    }
    
    /**
     * 获取某个页面配置的按钮列表
     *
     * @param menuId
     * @param roleId
     * @return
     */
    @GetMapping(value = "role/getBtnList")
    @ApiOperation(value = "获取某个页面配置的按钮列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "menuId", value = "菜单id", dataType = "String", paramType = "query", required = true),})
    public BaseResponse<Object> getPageBtnList(String menuId) {
        if (StringUtils.isEmpty(menuId)) {
            throw new ServiceException("缺少参数");
        }
        return new BaseResponse<Object>(roleService.getPageBtnList(menuId));
    }
    
    /**
     * 角色启用/禁用
     *
     * @param roleId
     * @param status
     * @return
     */
    @PostMapping(value = "role/updateStatus")
    @ApiOperation(value = "角色启用/禁用")
    @ApiImplicitParams({@ApiImplicitParam(name = "roleId", value = "角色id", dataType = "int", paramType = "query", required = true),
        @ApiImplicitParam(name = "status", value = "状态（0：禁用，1：正常）", dataType = "int", paramType = "query", required = true),})
    public BaseResponse<Object> updateStatus(Integer roleId, Integer status) {
        if (roleId == null || status == null) {
            throw new ServiceException("缺少参数");
        }
        roleService.updateStatus(roleId, status);
        return new BaseResponse<>();
    }
}
