package com.starcharge.wios.auth.service;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.dao.MenuMapper;
import com.starcharge.wios.auth.dao.model.Menu;
import com.starcharge.wios.auth.daoex.MenuMapperEx;
import com.starcharge.wios.auth.vo.AbsractTreeNode;
import com.starcharge.wios.auth.vo.MenuTreeVo;
import com.starcharge.wios.auth.vo.MenuVo;

/**
 * 菜单service
 * 
 * @author lilin
 * @version [版本号, 2020年4月29日]
 */
@Service
public class MenuService {
    /**
     * 菜单mapper
     */
    @Resource
    private MenuMapper menuMapper;
    
    /**
     * 菜单扩展mapper
     */
    @Resource
    private MenuMapperEx menuMapperEx;
    
    /**
     * 获取用户所有有效的菜单
     * 
     * @param accountId 用户id
     * @return 菜单列表
     */
    public List<Menu> findAccountMenuList(Integer accountId) {
        Assert.notNull(accountId, "accountId不能为空");
        return menuMapperEx.findAccountMenuList(accountId);
    }
    
    /**
     * 获取角色所有有效的菜单
     * 
     * @param roleId 角色id
     * @return 菜单列表
     */
    public List<Menu> findRoleMenuList(Integer roleId) {
        return menuMapperEx.findRoleMenuList(roleId);
    }
    
    /**
     * 将数据库中的menu转换为前端需要的菜单格式
     *
     * @param menuList 有序菜单列表
     * @param menuType 0：左侧权限菜单 1：角色菜单树
     * @return 前端需要的菜单格式
     */
    public List<AbsractTreeNode> convertMenuList(List<Menu> menuList, int menuType) {
        if (CollectionUtils.isEmpty(menuList)) {
            return null;
        }
        // id为空表示根节点
        return findChildMenuList(null, menuList, menuType);
    }
    
    private List<AbsractTreeNode> findChildMenuList(String id, List<Menu> menuList, int menuType) {
        // 子菜单
        List<AbsractTreeNode> childList = new ArrayList<>();
        // 因为是有序列表，为了减少遍历次数，往后截取列表，设置当前索引
        int index = 0;
        for (Menu menu : menuList) {
            index++;
            // 父菜单id的长度大于等于当前菜单的长度
            if (id != null && id.length() >= menu.getId().length()) {
                break;
            }
            // 遍历所有节点，将父菜单id与传过来的id比较
            if (StringUtils.isBlank(menu.getParentId()) || menu.getParentId().equals(id)) {
                AbsractTreeNode menuVo = converMenu(menu, menuType);
                childList.add(menuVo);
                menuVo.setChildTreeNode(findChildMenuList(menu.getId(), menuList.subList(index, menuList.size()), menuType));
            }
        }
        // 递归退出条件
        if (childList.isEmpty()) {
            return null;
        }
        return childList;
    }
    
    /**
     * 把menu转换成为menuVo
     *
     * @param menu menu
     * @param menuType 0：左侧权限菜单 1：角色菜单树
     * @return menuVo
     */
    private AbsractTreeNode converMenu(Menu menu, int menuType) {
        if (menuType == 0) {
            MenuVo menuVo = new MenuVo();
            menuVo.setId(menu.getId());
            menuVo.setIcon(menu.getIcon());
            menuVo.setName(menu.getcName());
            menuVo.setPath(menu.getPath());
            return menuVo;
        } else if (menuType == 1) {
            MenuTreeVo menuTreeVo = new MenuTreeVo();
            menuTreeVo.setKey(menu.getId());
            menuTreeVo.setIcon(menu.getIcon());
            menuTreeVo.setTitle(menu.getcName());
            menuTreeVo.setChecked(menu.getStatus() != null);
            return menuTreeVo;
        }
        return null;
    }
    
    /**
     * 查询按钮列表
     *
     * @param roleId
     * @param menuId
     * @return
     */
    public List<Menu> findBtnList(Integer roleId, String menuId) {
        return menuMapperEx.findBtnList(roleId, menuId);
    }
}
