package com.starcharge.wios.auth.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ihidea.core.support.SpringContextLoader;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.pageLimit.PageLimit;
import com.ihidea.core.support.pageLimit.PageLimitHolderFilter;
import com.starcharge.base.util.Assert;
import com.starcharge.wios.auth.bo.RoleBo;
import com.starcharge.wios.auth.dao.AccountRoleMapper;
import com.starcharge.wios.auth.dao.RoleMapper;
import com.starcharge.wios.auth.dao.RoleMenuMapper;
import com.starcharge.wios.auth.dao.model.AccountRole;
import com.starcharge.wios.auth.dao.model.AccountRoleCriteria;
import com.starcharge.wios.auth.dao.model.Menu;
import com.starcharge.wios.auth.dao.model.Role;
import com.starcharge.wios.auth.dao.model.RoleCriteria;
import com.starcharge.wios.auth.dao.model.RoleMenu;
import com.starcharge.wios.auth.dao.model.RoleMenuCriteria;
import com.starcharge.wios.auth.daoex.RoleMapperEx;
import com.starcharge.wios.auth.vo.AbsractTreeNode;
import com.starcharge.wios.auth.vo.MenuTreeVo;

/**
 * 角色Service
 * 
 * @author lilin
 * @version [版本号, 2020年5月20日]
 */
@Service
public class RoleService {
    /**
     * 角色Mapper
     */
    @Resource
    private RoleMapper roleMapper;
    
    /**
     * 账户service
     */
    @Resource
    private AccountService accountService;
    
    /**
     * 菜单service
     */
    @Resource
    private MenuService menuService;
    
    /**
     * 角色扩展Mapper
     */
    @Resource
    private RoleMapperEx roleMapperEx;
    
    @Autowired
    private TokenService tokenService;
    
    /**
     * 角色菜单Mapper
     */
    @Resource
    private RoleMenuMapper roleMenuMapper;
    
    @Autowired
    private AccountRoleMapper accountRoleMapper;
    
    /**
     * 查询角色列表
     *
     * @param query 查询条件
     * @return 角色列表
     */
    public List<Role> queryList(RoleBo role) {
        RoleCriteria roleCriteria = new RoleCriteria();
        RoleCriteria.Criteria criteria = roleCriteria.createCriteria();
        if (role.getStatus() != null)
            criteria.andStatusEqualTo(role.getStatus());
        criteria.andIdNotEqualTo(1);// 去除超级管理员
        if (role.getScene() != null && role.getScene().intValue() == 1) {
            criteria.andIdNotEqualTo(2);
            criteria.andIdNotEqualTo(3);
            criteria.andIdNotEqualTo(4);
        }
        
        PageLimit pageLimit = PageLimitHolderFilter.getContext();
        if (pageLimit != null)
            pageLimit.setLimited(true);
        Role camsPMRole = null;
        TokenService tokenService = (TokenService)SpringContextLoader.getBean("tokenService");
        if (tokenService != null && tokenService.getUser() != null) {
            int accountId = Integer.valueOf(tokenService.getUser().getUserId()).intValue();
            String roleIdActual = tokenService.getUser().getAttribute().get("roleIdActual");
            if (StringUtils.isNotEmpty(roleIdActual)) {
                if (Integer.valueOf(roleIdActual).intValue() == 6) {// cams管理员
                    camsPMRole = roleMapper.selectByPrimaryKey(5);// cams区域经理
                }
            }
            if (accountId != 1)
                criteria.andCreateAccountIdEqualTo(accountId);
        }
        
        List<Role> roleList = roleMapper.selectByExample(roleCriteria);
        if (camsPMRole != null)
            roleList.add(camsPMRole);
        if (pageLimit != null)
            return pageLimit.limitList(roleList);
        else
            return roleList;
    }
    
    /**
     * 新增角色
     *
     * @param role 角色信息
     * @return 新增成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int addRole(RoleBo role) {
        Assert.notEmpty(role.getName(), "角色名称不能为空");
        role.setCreateTime(new Date());
        role.setStatus(1); // 正常
        TokenService tokenService = (TokenService)SpringContextLoader.getBean("tokenService");
        if (tokenService != null && tokenService.getUser() != null) {
            role.setCreateAccountId(Integer.valueOf(tokenService.getUser().getUserId()));
            
//            List<Role> roleList = roleMapperEx.findRoleListByCreate(Integer.valueOf(tokenService.getUser().getUserId()));
//            roleList = roleList.stream().filter(r->r.getName().equals(role.getName())).collect(Collectors.toList());
//            if(!roleList.isEmpty()) throw new ServiceException("名称不能重复");
        }
        int result = roleMapper.insertSelective(role);
        
        if (StringUtils.isNotBlank(role.getMenuIds())) {
            // 补全上级菜单
            {
                String menuIds = role.getMenuIds();
                String[] menuArr = menuIds.split(",");
                List<String> menuList = new ArrayList<>(Arrays.asList(menuArr));
                for (String menu : menuArr) {
                    while (true) {
                        if (menu.length() > 2) {
                            menu = menu.substring(0, menu.length() - 2);
                            if (!menuList.contains(menu)) {
                                menuList.add(menu);
                            }
                        } else {
                            break;
                        }
                    }
                }
                role.setMenuIds(org.springframework.util.StringUtils.collectionToCommaDelimitedString(menuList));
            }
        }
        
        batchInsertRoleMenu(role.getId(), role.getMenuIds());
        return result;
    }
    
    /**
     * 修改角色
     *
     * @param role 角色信息
     * @return 修改成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int updateRole(RoleBo role) {
        Assert.notNull(role.getId(), "id不能为空");
        
        if (StringUtils.isNotBlank(role.getMenuIds())) {
            // 补全上级菜单
            {
                String menuIds = role.getMenuIds();
                String[] menuArr = menuIds.split(",");
                List<String> menuList = new ArrayList<>(Arrays.asList(menuArr));
                for (String menu : menuArr) {
                    while (true) {
                        if (menu.length() > 2) {
                            menu = menu.substring(0, menu.length() - 2);
                            if (!menuList.contains(menu)) {
                                menuList.add(menu);
                            }
                        } else {
                            break;
                        }
                    }
                }
                role.setMenuIds(org.springframework.util.StringUtils.collectionToCommaDelimitedString(menuList));
            }
            
            RoleMenuCriteria criteria = new RoleMenuCriteria();
            criteria.createCriteria().andRoleIdEqualTo(role.getId());
            roleMenuMapper.deleteByExample(criteria);
            batchInsertRoleMenu(role.getId(), role.getMenuIds());
        }
        if (!StringUtils.isEmpty(role.getName()) || !StringUtils.isEmpty(role.getRemark())) {
            return roleMapper.updateByPrimaryKeySelective(role);
        }
        return 1;
    }
    
    /**
     * 批量新增权限菜单
     *
     * @param roleId 角色id
     * @param menuIds 菜单id
     * @return 新增成功的条数
     */
    private int batchInsertRoleMenu(int roleId, String menuIds) {
        if (StringUtils.isBlank(menuIds)) {
            return 0;
        }
        TokenService tokenService = (TokenService)SpringContextLoader.getBean("tokenService");
        final Integer createAccountId = tokenService.getUser() == null ? null : Integer.valueOf(tokenService.getUser().getUserId());
        Date now = new Date();
        
        return roleMapperEx.batchInsertRoleMenu(Stream.of(menuIds.split(",")).map(menuId -> {
            RoleMenu roleMenu = new RoleMenu();
            roleMenu.setRoleId(roleId);
            roleMenu.setMenuId(menuId);
            roleMenu.setCreateAccountId(createAccountId);
            roleMenu.setCreateTime(now);
            return roleMenu;
        }).collect(Collectors.toList()));
    }
    
    /**
     * 删除角色
     *
     * @param id 角色id
     * @return 删除成功的数量
     */
    @Transactional(rollbackFor = Exception.class)
    public int deleteRole(Integer id) {
        Assert.notNull(id, "id不能为空");
        if (id == 1 || id == 2 || id == 3 || id == 4 || id == 5 || id == 6) {
            throw new ServiceException("内置角色不能被删除！");
        }
        
        AccountRoleCriteria example = new AccountRoleCriteria();
        example.createCriteria().andRoleIdEqualTo(id);
        List<AccountRole> arList = accountRoleMapper.selectByExample(example);
        if (!arList.isEmpty())
            throw new ServiceException("该角色已绑定账户，不可删除");
        
        RoleMenuCriteria criteria = new RoleMenuCriteria();
        criteria.createCriteria().andRoleIdEqualTo(id);
        roleMenuMapper.deleteByExample(criteria);
        
        return roleMapper.deleteByPrimaryKey(id);
    }
    
    /**
     * 查询角色详细信息
     *
     * @param id 角色id
     * @return 角色详细信息
     */
    // public RoleDetailVo getRoleDetail(Integer id) {
    // Role role = null;
    // if (id != null) {
    // role = roleMapper.selectByPrimaryKey(id);
    // }
    // RoleDetailVo roleDetailVo = new RoleDetailVo();
    //
    // if (role == null) {
    // role = new Role();
    // } else {
    // BeanUtils.copyProperties(role, roleDetailVo);
    // }
    //
    // List<Menu> menuList = menuService.findRoleMenuList(id);
    // roleDetailVo.setMenuTreeList(menuService.convertMenuList(menuList, 1));
    //
    // return roleDetailVo;
    // }
    
    /**
     * 查询角色菜单权限
     *
     * @param id 角色id
     * @return 角色菜单权限
     */
    public List<MenuTreeVo> getRoleMenuList(Integer id) {
        int currentAccountRoleId = Integer.valueOf(tokenService.getUser().getAttribute("roleIdActual"));
        List<Menu> currentAccountMenuList = menuService.findRoleMenuList(currentAccountRoleId);
        List<Menu> menuList = menuService.findRoleMenuList(id);
        
        for (Menu caMenu : currentAccountMenuList) {
            List<Menu> hasList = menuList.stream().filter(m -> m.getId().equals(caMenu.getId())).collect(Collectors.toList());
            if (hasList.isEmpty()) {
                caMenu.setStatus(null);
            } else {
                caMenu.setStatus(1);
            }
        }
        // TODO 判断如果不是管理员权限 去除角色管理的菜单
        {
            if (currentAccountRoleId != 1) {
                currentAccountMenuList =
                    currentAccountMenuList.stream().filter(menu -> !menu.getId().startsWith("0205")).collect(Collectors.toList());
            }
        }
        List<AbsractTreeNode> returnList = menuService.convertMenuList(currentAccountMenuList, 1);
        List<MenuTreeVo> returnValue = new ArrayList<>();
        for (AbsractTreeNode node : returnList) {
            MenuTreeVo vo = (MenuTreeVo)node;
            // vo.setChecked(judge(vo.getChildren()));
            returnValue.add(vo);
        }
        return returnValue;
    }
    
    public boolean judge(List<MenuTreeVo> children) {
        if (children == null)
            return true;
        boolean checked = true;
        for (MenuTreeVo vo : children) {
            if (!vo.isChecked()) {
                checked = false;
                break;
            } else {
                if (!judge(vo.getChildren())) {
                    return false;
                }
            }
        }
        return checked;
    }
    
    /**
     * 查询左侧菜单列表
     *
     * @param userId
     * @return
     */
    public List<AbsractTreeNode> getMenuList(Integer accountId) {
        List<Menu> menuList = menuService.findAccountMenuList(accountId);
        return menuService.convertMenuList(menuList, 0);
    }
    
    /**
     * 根据账户id查询角色列表
     *
     * @param accountId 账户id
     * @return 角色列表
     */
    public List<Role> findAccountRoleList(String accountId) {
        return roleMapperEx.findAccountRoleList(accountId);
    }
    
    /**
     * 根据账号id查询角色和小程序权限
     * 
     * @param accountId
     * @return
     */
    public List<RoleBo> findMinaRoleMenu(Integer accountId) {
        return roleMapperEx.findMinaRoleMenu(accountId);
    }
    
    /**
     * 获取某个页面配置的按钮列表
     *
     * @param menuId
     * @return
     */
    public List<Menu> getPageBtnList(String menuId, String accountId) {
        if (StringUtils.isEmpty(accountId)) {
            TokenService tokenService = (TokenService)SpringContextLoader.getBean("tokenService");
            if (tokenService.getUser() == null) {
                throw new ServiceException("获取用户信息失败，请重新登陆");
            }
            accountId = tokenService.getUser().getUserId();
        }
        List<Role> roleList = roleMapperEx.findAccountRoleList(accountId);
        List<Menu> menuList = new ArrayList<>();
        for (Role role : roleList) {
            List<String> ids = menuList.stream().map(item -> item.getId()).collect(Collectors.toList());
            List<Menu> list = menuService.findBtnList(role.getId(), menuId);
            list.stream().forEach(item -> {
                if (!ids.contains(item.getId())) {
                    menuList.add(item);
                }
            });
        }
        return menuList;
    }
    
    public List<Menu> getPageBtnList(String menuId) {
        return getPageBtnList(menuId, null);
    }
    
    /**
     * 角色启用/禁用
     *
     * @param roleId
     * @param status
     * @return
     */
    public void updateStatus(Integer roleId, Integer status) {
        Role record = new Role();
        record.setId(roleId);
        record.setStatus(status);
        roleMapper.updateByPrimaryKeySelective(record);
    }
    
}
