package com.starcharge.wios.auth.service;

import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.ihidea.core.support.servlet.ServletHolderFilter;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.wios.utils.SpringContextUtil;

/**
 * @author lilin
 * @date 2020-05-26
 * @description 处理前后端接口调用中的token信息
 */
@Service
public class TokenService {
    /**
     * token的redis前缀
     */
    public static final String TOKEN_KEY_PREFIX = "user:login:token:";
    
    /**
     * token默认失效时间
     */
    @Value("${auth.token.validSeconds:7200}")
    private Integer tokenValidSeconds;

    /**
     * mina token默认失效时间
     */
    @Value("${auth.token.minaValidSeconds:7200}")
    private Integer tokenMinaValidSeconds;

    /**
     * 微信公众号token默认失效时间-10天
     */
    @Value("${auth.token.wxValidSeconds:864000}")
    private Integer tokenWxValidSeconds;
    
    @Resource
    private RedisClient redisClient;
    
    /**
     * 保存用户session 信息
     *
     * @param sessionInfo 用户会话信息
     * @return token
     * @author xgl
     * @date 2020-05-26
     */
    public String saveToken(Object sessionInfo) {
        return saveToken(sessionInfo, tokenValidSeconds);
    }

    /**
     * 小程序保存token
     * @param sessionInfo
     * @return
     */
    public String saveMinaToken(Object sessionInfo) {
        return saveToken(sessionInfo, tokenMinaValidSeconds);
    }

    /**
     * 公众号H5保存token
     * @param sessionInfo
     * @return
     */
    public String saveWxToken(Object sessionInfo) {
        return saveToken(sessionInfo, tokenWxValidSeconds);
    }

    public String saveToken(Object sessionInfo, Integer expire) {
        String token = UUID.randomUUID().toString();
        redisClient.put(TOKEN_KEY_PREFIX + token, sessionInfo, expire);
        return token;
    }
    
    /**
     * 验证token
     *
     * @param token token
     * @return 是否验证通过
     * @author xgl
     * @date 2020-05-26
     */
    public boolean verifyToken(String token) {
        return verifyToken(token, tokenValidSeconds);
    }

    /**
     * 小程序验证token
     * @param token
     * @return
     */
    public boolean verifyMinaToken(String token) {
        return verifyToken(token, tokenMinaValidSeconds);
    }

    /**
     * 公众号h5验证token
     * @param token
     * @return
     */
    public boolean verifyWxToken(String token) {
        return verifyToken(token, tokenWxValidSeconds);
    }

    public boolean verifyToken(String token, Integer expire) {
        if (StringUtils.isBlank(token)) {
            return false;
        }
        SessionInfo user = redisClient.get(TOKEN_KEY_PREFIX + token, SessionInfo.class, expire);
        if (user == null) {
            return false;
        }
        return true;
    }
    
    /**
     * 获取SessionInfo
     *
     * @return 用户会话信息
     * @author xgl
     * @date 2020-05-26
     */
    public SessionInfo getUser() {
        String token = null;
        if (ServletHolderFilter.getContext() == null || ServletHolderFilter.getContext().getRequest().getHeader("Authorization") == null) {
            token = SpringContextUtil.getToken();
        }else{
            token = ServletHolderFilter.getContext().getRequest().getHeader("Authorization");
        }
        return getUser(token);
    }
    
    /**
     * 根据token获取SessionInfo
     *
     * @param token token值
     * @return 用户会话信息
     * @author xgl
     * @date 2020-05-26
     */
    public SessionInfo getUser(String token) {
        if (StringUtils.isEmpty(token)) {
            return null;
        }
        return redisClient.get(TOKEN_KEY_PREFIX + token, SessionInfo.class);
    }

    /**
     * token失效
     * @param token
     */
    public void invalidToken(String token){
        if (StringUtils.isEmpty(token)) {
            redisClient.remove(TOKEN_KEY_PREFIX + token);
        }
    }
    
}
