package com.starcharge.wios.service;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.bo.OperateLogBo;
import com.starcharge.wios.dao.OperateLogDao;
import com.starcharge.wios.dao.OrderDao;
import com.starcharge.wios.dao.OrderScoreDao;
import com.starcharge.wios.dao.entity.OperateLog;
import com.starcharge.wios.dao.entity.Order;
import com.starcharge.wios.dao.entity.OrderScore;
import com.starcharge.wios.dto.OrderScoreDTO;
import com.starcharge.wios.dto.OrderScoreLessThan60DTO;
import com.starcharge.wios.dto.ScoreSupplierDTO;
import com.starcharge.wios.enums.OrderStatus;

import cn.hutool.core.date.DateUtil;

/**
 * 供应商评分的明细
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@Service
public class OrderScoreService {

    @Autowired
    private OrderScoreDao orderScoreDao;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private ScoreSupplierService scoreSupplierService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private OrderDao orderDao;

    /**
     * 根据操作日志判断客户是否已经评价（回访）
     * @param orderId
     * @return
     */
    public boolean isOrderScore(String orderId){
        return getOrderScoreLog(orderId) != null;
    }

    public OperateLog getOrderScoreLog(String orderId){
        OperateLogBo operateLogQuery = new OperateLogBo();
        operateLogQuery.setThirdPartyId(orderId);
        operateLogQuery.setName(String.valueOf(OrderStatus.RETURN_VISIT.getType()));
        List<OperateLog> operateLogList = operateLogDao.selectByExample(operateLogQuery);
        return CollectionUtils.isNotEmpty(operateLogList) ? operateLogList.get(0) : null;
    }

    public OrderScoreDTO getOrderScoreByOrderId(String orderId){
        OrderScoreDTO orderScoreDTO = new OrderScoreDTO();
        orderScoreDTO.setOrderId(orderId);
        OperateLogBo operateLogQuery = new OperateLogBo();
        operateLogQuery.setThirdPartyId(orderId);
        operateLogQuery.setName("61");
        List<OperateLog> operateLogList = operateLogDao.selectByExample(operateLogQuery);
        if(operateLogList.size() > 0){
            orderScoreDTO.setVisitContent(operateLogList.get(0).getDesc());
            orderScoreDTO.setVisitorId(operateLogList.get(0).getCreateAccount());
            orderScoreDTO.setVisitorName(operateLogList.get(0).getCreateAccountName());
            orderScoreDTO.setVisitTime(operateLogList.get(0).getCreateTime());

            Order order=orderDao.selectById(orderId);
            if(order!=null){
                orderScoreDTO.setFiles(order.getFiles());
            }
            orderScoreDTO.setHasScore(Boolean.TRUE);
        }else {
            orderScoreDTO.setVisitContent("");
            SessionInfo user = tokenService.getUser();
            if (user != null) {
                orderScoreDTO.setVisitorId(Integer.valueOf(user.getUserId()));
                orderScoreDTO.setVisitorName(user.getUserName());
                orderScoreDTO.setVisitTime(DateUtil.date());
                orderScoreDTO.setHasScore(Boolean.FALSE);
            }
        }
        orderScoreDTO.setOrderScoreList(this.getListByOrderId(orderId));

        return orderScoreDTO;
    }

    /**
     * 查询明细
     *
     * @param orderId
     * @return
     */
    public List<OrderScore> getListByOrderId(String orderId){
        List<OrderScore> list = orderScoreDao.getListByOrderId(orderId);
        if(list.size() ==0){
            //如果没有记录，则增加相应的客户评分记录，初始化
            orderScoreDao.insertOrderScore(orderId);
            list = orderScoreDao.getListByOrderId(orderId);
        }
        return list;
    }

    /**
     * 单个更新
     *
     * @param orderScore
     */
    public void update(OrderScore orderScore) {
        if(orderScore.getId() == null || orderScore.getId()==0){
            throw new ServiceException("没有该条记录的 ID");
        }
        orderScoreDao.update(orderScore);
    }

    /**
     * 通过编码和主表id更新明细表记录
     *
     * @param orderScore
     */
    public void updateByCode(OrderScore orderScore) {
        if(StringUtils.isEmpty(orderScore.getCode()) || orderScore.getOrderId() ==null){
            throw new ServiceException("编码和订单表ID不能为空！");
        }
        orderScoreDao.updateByCode(orderScore);
    }

    /**
     * 评分小于60的订单，用于评分报表
     * @param scoreSupplierId
     * @return
     */
    public List<OrderScoreLessThan60DTO> getListLessThan60(Long scoreSupplierId){
        List<OrderScoreLessThan60DTO> result = new ArrayList<OrderScoreLessThan60DTO>();
        ScoreSupplierDTO scoreSupplier = scoreSupplierService.selectById(scoreSupplierId);
        List<Order> orderList = orderScoreDao.getOrderListLessThan60(scoreSupplier.getSupplierId(),scoreSupplier.getRegionId(), scoreSupplier.getMonth());

        for(Order order : orderList){
            List<OrderScore> orderScoreList = orderScoreDao.getListByOrderId(order.getId());
            for(OrderScore orderScore : orderScoreList){
                if (orderScore.getScore() != null && orderScore.getScore().doubleValue() < 60) {
                    OrderScoreLessThan60DTO orderScoreLessThan60DTO = new OrderScoreLessThan60DTO();
                    orderScoreLessThan60DTO.setOrderId(order.getId());
                    orderScoreLessThan60DTO.setOrderScore(order.getScore());
                    orderScoreLessThan60DTO.setOrderFiles(order.getFiles());
                    orderScoreLessThan60DTO.setCode(orderScore.getCode());
                    orderScoreLessThan60DTO.setName(orderScore.getName());
                    orderScoreLessThan60DTO.setScore(orderScore.getScore());
                    orderScoreLessThan60DTO.setWeight(orderScore.getWeight());
                    result.add(orderScoreLessThan60DTO);
                }
            }
        }
        return result;
    }

}
