package com.starcharge.wios.service;

import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.bo.OrderBo;
import com.starcharge.wios.bo.SupplierContractBo;
import com.starcharge.wios.contants.ScoreConstant;
import com.starcharge.wios.dao.*;
import com.starcharge.wios.dao.entity.*;
import com.starcharge.wios.dto.ScoreSupplierDTO;
import com.starcharge.wios.dto.ScoreSupplierReportDTO;
import com.starcharge.wios.enums.ContractStatus;
import com.starcharge.wios.enums.OrderStatus;
import com.starcharge.wios.enums.ScoreStatus;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.vo.SupplierContractVo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.OptionalDouble;
import java.util.stream.Collectors;

/**
 * 供应商评分主表应用服务
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@Service
public class ScoreSupplierService {

    @Autowired
    private ScoreSupplierDao scoreSupplierDao;
    
    @Autowired
    private SupplierContractDao supplierContractDao;
    
    @Autowired
    private ScoreSupplierDetailService scoreSupplierDetailService;
    
    @Autowired
    private OrderScoreService orderScoreService;
    
    @Autowired
    private ScoreSupplierDetailDao scoreSupplierDetailDao;
    
    @Autowired
    private OrderDao orderDao;
    
    @Autowired
    private OrderScoreDao orderScoreDao;
    
    @Autowired
    private ScoreConfigDao scoreConfigDao;
    @Autowired
    private RegionDao regionDao;
    
    /**
     * 增加评分主表记录
     * 
     * @param scoreSupplier
     * @return
     */
    public int insert(ScoreSupplier scoreSupplier) {
        return scoreSupplierDao.insert(scoreSupplier);
    }
    
    /**
     * 删除供应商评分记录，假删除
     * 
     * @param id
     * @return
     */
    public int delete(Long id) {
        ScoreSupplier scoreSupplier = new ScoreSupplier();
        scoreSupplier.setId(id);
        scoreSupplier.setStatus(ScoreStatus.INVALID.getType());
        return this.update(scoreSupplier);
    }
    
    /**
     * 取服务商评价详情信息
     * 
     * @param scoreSupplierId
     * @return
     */
    public ScoreSupplierReportDTO detail(Long scoreSupplierId) {
        ScoreSupplierReportDTO result = new ScoreSupplierReportDTO();
        ScoreSupplierDTO scoreSupplierDTOQuery = new ScoreSupplierDTO();
        scoreSupplierDTOQuery.setId(scoreSupplierId);
        ScoreSupplierDTO dto = scoreSupplierDao.selectById(scoreSupplierId);
        ScoreConfig sc1 = scoreConfigDao.selectByCode(ScoreConstant.FINAL_PURCHASING);
        if (sc1 != null)
            dto.setPurchasingWeight(sc1.getWeight());
        ScoreConfig sc2 = scoreConfigDao.selectByCode(ScoreConstant.FINAL_ENGINEERING);
        if (sc2 != null)
            dto.setEngineeringWeight(sc2.getWeight());
        result.setScoreSupplierDTO(dto);
        
        // 取评分明细
        ScoreSupplierDetail scoreSupplierDetailQuery = new ScoreSupplierDetail();
        scoreSupplierDetailQuery.setScoreSupplierId(scoreSupplierId);
        result.setScoreSupplierDetailList(scoreSupplierDetailService.selectByExample(scoreSupplierDetailQuery));
        
        // 取低评分的订单
        result.setOrderScoreLessThan60DTOList(orderScoreService.getListLessThan60(scoreSupplierId));
        
        return result;
    }
    
    /**
     * 根据id取供应商某个区域的评分记录
     * 
     * @param id
     * @return
     */
    public ScoreSupplierDTO selectById(Long id) {
        return scoreSupplierDao.selectById(id);
    }
    
    /**
     * 查询列表
     *
     * @param scoreSupplierDTO
     * @return
     */
    public List<ScoreSupplierDTO> getList(ScoreSupplierDTO scoreSupplierDTO) {
        ScoreConfig sc1 = scoreConfigDao.selectByCode(ScoreConstant.FINAL_PURCHASING);
        ScoreConfig sc2 = scoreConfigDao.selectByCode(ScoreConstant.FINAL_ENGINEERING);
        List<ScoreSupplierDTO> list = scoreSupplierDao.selectByExample(scoreSupplierDTO);
        for (ScoreSupplierDTO dto : list) {
            if (sc1 != null)
                dto.setPurchasingWeight(sc1.getWeight());
            if (sc2 != null)
                dto.setEngineeringWeight(sc2.getWeight());
        }
        return list;
    }
    
    /**
     * 更新供应商评分的主表记录
     *
     * @param scoreSupplier
     */
    public int update(ScoreSupplier scoreSupplier) {
        if (scoreSupplier.getId() == null || scoreSupplier.getId() == 0) {
            throw new ServiceException("没有该条记录的 ID");
        }
        return scoreSupplierDao.update(scoreSupplier);
    }
    
    /**
     * 根据服务商id取最新的采购评分
     * 
     * @param supplierId
     * @return
     */
    public List<ScoreSupplierDetail> getPurchasingScoreList(Long supplierId) {
        ScoreSupplierDTO scoreSupplierDTO = new ScoreSupplierDTO();
        scoreSupplierDTO.setSupplierId(supplierId);
        scoreSupplierDTO.setStatus(ScoreStatus.PROCESSING.getType());
        List<ScoreSupplierDTO> scoreSupplierDTOList = scoreSupplierDao.selectByExample(scoreSupplierDTO);
        if (scoreSupplierDTOList.size() > 0) {
            ScoreSupplierDetail query = new ScoreSupplierDetail();
            query.setScoreSupplierId(scoreSupplierDTOList.get(0).getId());
            query.setParentCode(ScoreConstant.FINAL_PURCHASING);
            return scoreSupplierDetailService.selectByExample(query);
        } else {
            throw new ServiceException("该服务商没有待确定的采购评分");
        }
    }
    
    /**
     * 更新采购评分，如有多区域，则同时更新
     * 
     * @param list
     * @return
     */
    public int updatePurchasingScore(List<ScoreSupplierDetail> list) {
        if (list.size() > 0) {
            ScoreSupplierDetail ssd = scoreSupplierDetailDao.selectById(list.get(0).getId().intValue());
            ScoreSupplier scoreSupplier = scoreSupplierDao.selectById(ssd.getScoreSupplierId());
            ScoreSupplierDTO scoreSupplierDTO = new ScoreSupplierDTO();
            scoreSupplierDTO.setSupplierId(scoreSupplier.getSupplierId());
            scoreSupplierDTO.setMonth(scoreSupplier.getMonth());
            List<ScoreSupplierDTO> scoreSupplierDTOList = scoreSupplierDao.selectByExample(scoreSupplierDTO);
            for (ScoreSupplierDTO item : scoreSupplierDTOList) {
                for (ScoreSupplierDetail detail : list) {
                    ScoreSupplierDetail ssd1 = scoreSupplierDetailDao.selectById(detail.getId().intValue());
                    ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
                    scoreSupplierDetail.setScore(detail.getScore());
                    scoreSupplierDetail.setScoreSupplierId(item.getId());
                    // if (ScoreConstant.PURCHASING_ITEM1.equals(detail.getCode()) ||
                    // ScoreConstant.PURCHASING_ITEM2.equals(detail.getCode())
                    // || ScoreConstant.PURCHASING_ITEM3.equals(detail.getCode())) {
                    scoreSupplierDetail.setCode(ssd1.getCode());
                    scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
                    // }
                }
            }
        }
        return list.size();
    }
    
    /**
     * 开始供应商评分，系统供应商合同产生初始数据，对相关评分项季度做统计，每月21日始自动运行
     * 
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int beginScoreSupplier() {
        String month = this.getLastMonth();
        SupplierContractBo supplierContractBo = new SupplierContractBo();
        supplierContractBo.setStatus(ContractStatus.EFFECTIVE.getType());
        List<SupplierContractVo> list = supplierContractDao.getList(supplierContractBo);
        for (SupplierContractVo contract : list) {
            if (!StringUtils.isEmpty(contract.getRegionId())) {
                String[] regionArr = contract.getRegionId().split(",");
                for (String regionId : regionArr) {
                    // 新增供应商评分的主表记录
                    ScoreSupplier scoreSupplier = new ScoreSupplier();
                    scoreSupplier.setSupplierId(Long.parseLong(contract.getSupplierId()));
                    scoreSupplier.setRegionId(Long.parseLong(regionId));
                    scoreSupplier.setMonth(month);
                    scoreSupplierDao.insert(scoreSupplier);
                }
                
            }
        }
        if (list.size() > 0) {
            // 取本次新增的评分表
            ScoreSupplierDTO query = new ScoreSupplierDTO();
            query.setMonth(month);
            List<ScoreSupplierDTO> scoreSupplierDTOList = scoreSupplierDao.selectByExample(query);
            for (ScoreSupplierDTO scoreSupplierDTO : scoreSupplierDTOList) {
                Long scoreSupplierId = scoreSupplierDTO.getId();
                List<ScoreSupplierDetail> detailList = scoreSupplierDetailService.getListByScoreSupplierId(scoreSupplierId);
                if (detailList.size() == 0) {
                    // 新增供应商评分的明细记录
                    scoreSupplierDao.insertMonthDetail(scoreSupplierId);
                }
                
                // 统计某些评分项，作为人工评分的参考，或自动评分值
                this.setResultCalc(scoreSupplierId);
            }
            
        }
        
        return list.size();
    }
    
    /**
     * 计算某个评分表的统计数据
     * 
     * @return
     */
    public int setResultCalc(Long scoreSupplierId) {
        ScoreSupplier scoreSupplier = scoreSupplierDao.selectById(scoreSupplierId);
        Long supplierId = scoreSupplier.getSupplierId();
        Long regionId = scoreSupplier.getRegionId();
        String month = scoreSupplier.getMonth();
        ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
        scoreSupplierDetail.setScoreSupplierId(scoreSupplierId);
        
        // 如有上次采购评分，则取上次评分
        ScoreSupplier scoreSupplierLast = scoreSupplierDao.selectLastBySupplierId(scoreSupplier.getSupplierId());
        if (scoreSupplierLast != null) {
            List<ScoreSupplierDetail> lastDetailList = scoreSupplierDetailService.getListByScoreSupplierId(scoreSupplierLast.getId());
            for (ScoreSupplierDetail item : lastDetailList) {
                if (ScoreConstant.PURCHASING_ITEM1.equals(item.getCode())) {
                    scoreSupplierDetail.setCode(ScoreConstant.PURCHASING_ITEM1);
                    scoreSupplierDetail.setScore(item.getScore());
                    scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
                } else if (ScoreConstant.PURCHASING_ITEM2.equals(item.getCode())) {
                    scoreSupplierDetail.setCode(ScoreConstant.PURCHASING_ITEM2);
                    scoreSupplierDetail.setScore(item.getScore());
                    scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
                } else if (ScoreConstant.PURCHASING_ITEM3.equals(item.getCode())) {
                    scoreSupplierDetail.setCode(ScoreConstant.PURCHASING_ITEM3);
                    scoreSupplierDetail.setScore(item.getScore());
                    scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
                } else {
                    continue;
                }
            }
        }

        String threeMonthBefore = this.getThreeMonthBefore();
        // 统计更换服务商安装次数，作为扣分项1
        int deduct1 = scoreSupplierDao.countChangeSupplier(supplierId, threeMonthBefore,month);
        scoreSupplierDetail.setCode(ScoreConstant.DEDUCT_ITEM1);
        scoreSupplierDetail.setResultCalc(Double.valueOf(deduct1));
        scoreSupplierDetailService.updateByCode(scoreSupplierDetail);

        Region region = regionDao.selectById(regionId.intValue());
        // 订单完成率
        //派单量-关闭订单数量
        int countDispatchMinusClosedOrder =scoreSupplierDao.countOrderByCondition(supplierId, region.getIdTree(),OrderStatus.CLOSED.getType(),threeMonthBefore,month);
        //安装审核通过+安装完成待审核+安装完成+回访
        int countOrderFinished = scoreSupplierDao.countOrderHasFinished(supplierId, region.getIdTree(),threeMonthBefore,month);
        if (countDispatchMinusClosedOrder > 0) {
            scoreSupplierDetail.setCode(ScoreConstant.ONLINE_ITEM1);
            scoreSupplierDetail.setResultCalc(NumberUtil.roundDouble(countOrderFinished * 100.0 / countDispatchMinusClosedOrder));
            scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
        }

        // T1-安装时效
        //派单数量
        int countFinishedOrder =scoreSupplierDao.countOrderFinished(supplierId, region.getIdTree(), threeMonthBefore,month);
        //提交安装资料时间（安装完成时间）-派单时间
        Integer orderFinishedTime = scoreSupplierDao.countOrderFinishedTime(supplierId, region.getIdTree(), threeMonthBefore,month);
        if (countFinishedOrder>0) {
            scoreSupplierDetail.setCode(ScoreConstant.ONLINE_ITEM2);
            scoreSupplierDetail.setResultCalc(NumberUtil.roundDouble(Double.valueOf(orderFinishedTime==null?0:orderFinishedTime)/countFinishedOrder));
            scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
        }
        
        // T2-接单时效
        //预约勘察操作时间-派单时间
        int countOrderResponse =scoreSupplierDao.countOrderResponse(supplierId, region.getIdTree(), threeMonthBefore,month);
        Integer orderResponseTime = scoreSupplierDao.countOrderResponseTime(supplierId, region.getIdTree(), threeMonthBefore,month);
        if (countOrderResponse>0) {
            scoreSupplierDetail.setCode(ScoreConstant.ONLINE_ITEM3);
            scoreSupplierDetail.setResultCalc(NumberUtil.roundDouble(Double.valueOf(orderResponseTime==null?0:orderResponseTime)/countOrderResponse));
            scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
        }
        
        // T3勘测时效
        //提交勘测资料时间（勘测完成时间）-派单时间
        int countOrderSurvey =scoreSupplierDao.countOrderSurvey(supplierId, region.getIdTree(), threeMonthBefore,month);
        Integer orderSurveyTime = scoreSupplierDao.countOrderSurveyTime(supplierId, region.getIdTree(), threeMonthBefore,month);
        if (countOrderSurvey>0) {
            scoreSupplierDetail.setCode(ScoreConstant.ONLINE_ITEM4);
            scoreSupplierDetail.setResultCalc(NumberUtil.roundDouble(Double.valueOf(orderSurveyTime==null?0:orderSurveyTime)/countOrderSurvey));
            scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
        }
        return 0;
    }
    
    /**
     * 结束供应商评分，系统根据权重配置进行最终评分的技术安，每月7日终自动运行
     * 
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public int endScoreSupplier() {
        String month = this.getLastMonth();
        
        // 取本月度的评分表
        ScoreSupplierDTO query = new ScoreSupplierDTO();
        query.setMonth(month);
        List<ScoreSupplierDTO> scoreSupplierDTOList = scoreSupplierDao.selectByExample(query);
        for (ScoreSupplier scoreSupplier : scoreSupplierDTOList) {
            // 计算某个评分表的最终评分数据
            this.setFinalScore(scoreSupplier);
        }
        
        return scoreSupplierDTOList.size();
    }
    
    /**
     * 计算某个评分表的最终评分数据
     * 
     * @return
     */
    public int setFinalScore(ScoreSupplier scoreSupplierParm) {
        String threeMonthBefore = this.getThreeMonthBefore();
        Long scoreSupplierId = scoreSupplierParm.getId();
        // 计算客户评分细项平均值
        // if (scoreSupplierParm.getSupplierId().intValue() == 30) {
        // System.out.println("=================");
        // }
        Region region = regionDao.selectById(scoreSupplierParm.getRegionId().intValue());
        OrderBo condition = new OrderBo();
        condition.setInstallSupplierId(scoreSupplierParm.getSupplierId().intValue());
        condition.setRegionIdTree(region.getIdTree());
        condition.setQuarterLastMonth(scoreSupplierParm.getMonth().replaceAll("-", ""));
        condition.setQuarterFirstMonth(threeMonthBefore.replaceAll("-", ""));
        //取季度数据
        List<Order> orderList = orderDao.selectAll(condition);
        List<String> orderIds = orderList.stream().map(Order::getId).collect(Collectors.toList());
        if (!orderIds.isEmpty()) {
            List<OrderScore> orderScoreList = orderScoreDao.selectByTypeAndOrderIds(1, orderIds);
            List<ScoreSupplierDetail> ssdList =
                scoreSupplierDetailDao.getListByScoreSupplierIdAndPCode(scoreSupplierId, ScoreConstant.CUSTOMER);
            for (ScoreSupplierDetail ssd : ssdList) {
                String code = ssd.getCode();
                OptionalDouble averScore = orderScoreList.stream().filter(o -> o.getCode().equals(code) && o.getScore() != null)
                    .mapToDouble(OrderScore::getScore).average();
                Double score = null;
                if (averScore.isPresent())
                    score = averScore.getAsDouble();
                if (score != null) {
                    ScoreSupplierDetail updateSSD = new ScoreSupplierDetail();
                    updateSSD.setId(ssd.getId());
                    updateSSD.setScore(score);
                    scoreSupplierDetailDao.update(updateSSD);
                }
            }
        }
        
        List<ScoreSupplierDetail> list = scoreSupplierDetailService.getListByScoreSupplierId(scoreSupplierId);
        ScoreSupplier scoreSupplier = new ScoreSupplier();
        scoreSupplier.setId(scoreSupplierId);
        
        // 计算采购最后评分
        Double sumScore = this.calcScoreByParentCode(ScoreConstant.FINAL_PURCHASING, list);
        scoreSupplier.setFinalPurchasing(sumScore);
        
        // 计算工程线上评分
        sumScore = this.calcScoreByParentCode(ScoreConstant.ONLINE, list);
        scoreSupplier.setOnline(sumScore);
        
        // 计算工程线下评分
        sumScore = this.calcScoreByParentCode(ScoreConstant.OFFLINE, list);
        scoreSupplier.setOffline(sumScore);
        
        // 计算工程客户评分
        this.calcScoreByParentCode(ScoreConstant.CUSTOMER, list);
        sumScore = scoreSupplierDao.calcOrderScore(scoreSupplierParm.getSupplierId(), region.getIdTree(),threeMonthBefore,
            scoreSupplierParm.getMonth());
        scoreSupplier.setCustomer(sumScore);
        // 修改明细表中客户评分的值
        ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
        scoreSupplierDetail.setCode(ScoreConstant.CUSTOMER);
        scoreSupplierDetail.setScoreSupplierId(scoreSupplierId);
        scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
        
        // 计算工程最后评分
        sumScore = this.calcScoreByParentCode(ScoreConstant.FINAL_ENGINEERING, list);
        scoreSupplier.setFinalEngineering(sumScore);
        
        // 计算扣分项
        sumScore = this.calcScoreByParentCode(ScoreConstant.FINAL_DEDUCT, list);
        scoreSupplier.setFinalDeduct(sumScore);
        
        // 计算最终评分
        Double finalScore = 0.0;
        scoreSupplier.setFinalScore(finalScore);
        for (ScoreSupplierDetail item : list) {
            if (ScoreConstant.FINAL_PURCHASING.equals(item.getCode()) || ScoreConstant.FINAL_ENGINEERING.equals(item.getCode())) {
                finalScore += item.getScore() * item.getWeight() / 100.0;
            } else if (ScoreConstant.FINAL_DEDUCT.equals(item.getCode())) {
                finalScore -= item.getScore();
            }
        }
        scoreSupplier.setFinalScore(finalScore);
        
        // 修改主表
        scoreSupplier.setStatus(ScoreStatus.FINISHED.getType());
        scoreSupplierDao.update(scoreSupplier);
        
        return 0;
    }
    
    /**
     * 计算某一父项的评分
     * 
     * @param parentCode
     * @param list
     */
    protected Double calcScoreByParentCode(String parentCode, List<ScoreSupplierDetail> list) {
        Double sumScore = 0.0;
        for (ScoreSupplierDetail item : list) {
            // if (item.getScoreSupplierId().intValue() == 261 && item.getCode().startsWith("customer_item")) {
            // System.out.println("=========");
            // }
            if (parentCode.equals(item.getParentCode())) {
                if (parentCode.equals(ScoreConstant.FINAL_DEDUCT)) {
                    item.setWeight(100D);// 扣分项按次数计算 不算权重
                }
                // sumScore += item.getScore() * (item.getWeight() / 100.0);
                BigDecimal score = new BigDecimal(item.getScore()).multiply(new BigDecimal(item.getWeight()).divide(new BigDecimal(100)));
                sumScore = new BigDecimal(sumScore).add(score).doubleValue();
            }
        }
        // Long scoreSupplierId = 0L;
        // 设置父节点的值
        for (ScoreSupplierDetail item : list) {
            if (parentCode.equals(item.getCode())) {
                item.setScore(sumScore);
            }
        }
        // 修改父节点的值
        if (CollectionUtils.isNotEmpty(list)) {
            if (list.get(0).getScoreSupplierId() != null) {
                ScoreSupplierDetail scoreSupplierDetail = new ScoreSupplierDetail();
                scoreSupplierDetail.setCode(parentCode);
                scoreSupplierDetail.setScoreSupplierId(list.get(0).getScoreSupplierId());
                scoreSupplierDetail.setScore(sumScore);
                scoreSupplierDetailService.updateByCode(scoreSupplierDetail);
            }
        }
        return NumberUtil.roundDouble(sumScore);
    }
    
    /**
     * 获取上个月份的字符串
     * 
     * @return
     */
    protected String getLastMonth() {
        LocalDate today = LocalDate.now();
        today = today.minusMonths(1);
        DateTimeFormatter formatters = DateTimeFormatter.ofPattern("yyyy-MM");
        return formatters.format(today);
    }

    /**
     * 获取每季度第一个月
     *
     * @return
     */
    protected String getThreeMonthBefore() {
        LocalDate today = LocalDate.now();
        today = today.minusMonths(3);
        DateTimeFormatter formatters = DateTimeFormatter.ofPattern("yyyy-MM");
        return formatters.format(today);
    }
    
    // public void doAfterOrderScore(int supplierId, int regionId) {
    // OrderBo condition = new OrderBo();
    // condition.setInstallSupplierId(supplierId);
    // condition.setRegionId(regionId + "");
    // List<Order> orderList = orderDao.selectAll(condition);
    // orderList = orderList.stream().filter(order -> order.getScore() != null).collect(Collectors.toList());
    // if (!orderList.isEmpty()) {
    // List<String> orderIds = orderList.stream().map(Order::getId).collect(Collectors.toList());
    // List<OrderScore> orderScoreList = orderScoreDao.selectByTypeAndOrderIds(1, orderIds);
    // List<ScoreConfig> scoreConfigList = scoreConfigDao.selectByPCode(ScoreConstant.CUSTOMER);
    // for(ScoreConfig config : scoreConfigList){
    //
    // }
    // }
    // }
}
