package com.starcharge.wios.service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.starcharge.wios.dao.entity.Order;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.JSONUtilsEx;
import com.ihidea.core.util.StringUtilsEx;
import com.starcharge.base.redis.RedisClient;
import com.starcharge.base.util.Assert;
import com.starcharge.base.util.NetUtil;
import com.starcharge.component.pay.weixin.WeixinPayCore;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.dao.UserPayBillDao;
import com.starcharge.wios.dao.entity.OrderInstall;
import com.starcharge.wios.dao.entity.UserPayBill;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.utils.SpringContextUtil;
import com.starcharge.wios.vo.OrderInstallVo;
import com.starcharge.wios.vo.OrderVo;
import com.starcharge.wios.vo.UserPayBillVo;

import javax.annotation.Resource;

/**
 * @author kevin
 * @create 2020/11/16 15:38
 */
@Service
public class UserPayBillService {
    private static final Logger logger= LoggerFactory.getLogger(UserPayBillService.class);

    @Autowired
    private UserPayBillDao userPayBillDao;

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderInstallService orderInstallService;

    @Autowired
    private TokenService tokenService;

    private static final String PAY_QRCODE_CAHCE_KEY = "payQrcode:";

    public List<UserPayBill> list(UserPayBill entity){
        return userPayBillDao.list(entity);
    }
    
    public List<UserPayBillVo> selectList(UserPayBill entity) {
        return userPayBillDao.selectList(entity);
    }
    
    public List<UserPayBillVo> selectListForApp(UserPayBill entity) {
        return userPayBillDao.selectListForApp(entity);
    }

    public UserPayBill get(UserPayBill entity){
        List<UserPayBill> list=userPayBillDao.list(entity);
        return CollectionUtils.isNotEmpty(list) ? list.get(0) : null;
    }

    public UserPayBill getByRefId(String refId){
        Assert.notBlank(refId, "关联编号不能为空");
        UserPayBill entity=new UserPayBill();
        entity.setRefId(refId);
        return get(entity);
    }

    public Integer countByRefId(String refId){
        Assert.notBlank(refId, "关联编号不能为空");
        UserPayBill entity=new UserPayBill();
        entity.setRefId(refId);
        return userPayBillDao.count(entity);
    }

    /**
     * 根据订单编号生成账单，增项费用+报装费
     * @param orderId
     * @return
     */
    public UserPayBill generateBillByOrderId(String orderId){
        OrderVo orderVo = orderService.selectOwnOrderById(orderId);
        Assert.notNull(orderVo, "订单不存在");
        Assert.isTrue(StringUtils.equals(orderVo.getSupplierLevel(), "A"), "非自建服务商不能生成账单");

        OrderInstallVo orderInstallVo = orderInstallService.getByOrderId(orderId);
        Assert.notNull(orderInstallVo, "安装订单不存在");

        BigDecimal predictTotalFee = orderInstallVo.getPredictTotalFee() == null ? BigDecimal.ZERO : orderInstallVo.getPredictTotalFee();
        BigDecimal expandFee = orderInstallVo.getExpandFee() == null ? BigDecimal.ZERO : orderInstallVo.getExpandFee();
        BigDecimal totalFee = predictTotalFee.add(expandFee);
        Assert.isTrue(totalFee.compareTo(BigDecimal.ZERO) > 0, "支付金额必须大于0");

        UserPayBill bill = getByRefId(orderId);
        Assert.isNull(bill, "账单已存在，请勿重复生成");

        bill = new UserPayBill();
        bill.setId(NumberUtil.getRandomOrderId());
        bill.setRefId(orderId);
        String userPhone = orderVo.getInstallCamsPhone();
        if(StringUtils.isBlank(userPhone)){
            userPhone = orderVo.getSurveyCamsPhone();
        }
        if(StringUtils.isBlank(userPhone)){
            userPhone = orderVo.getUserPhone();
        }
        bill.setUserPhone(userPhone);
        bill.setAmount(totalFee);
        bill.setBusinessType(1);
        Map<String, String> infoObj = new HashMap<>();
        infoObj.put("predictTotalFee", predictTotalFee.toString());
        infoObj.put("expandFee", expandFee.toString());
        bill.setInfo(JSONUtilsEx.serialize(infoObj));
        add(bill);

        return bill;
    }

    /**
     * 根据工单编号生成付款二维码
     * @param orderId
     * @return
     */
    public String genPayQrcode(String orderId){
        String cacheKey = getQrcodeCacheKey(orderId);
//        String codeUrl = redisClient.get(cacheKey, String.class);
        String codeUrl = null;
        if(StringUtils.isBlank(codeUrl)) {
            // 判断当前订单账单是否存在且已支付
            UserPayBill bill = getByRefId(orderId);
            if (bill != null) {
                Assert.isTrue(bill.getStatus() == 0, "账单已支付");
            } else {
                bill = generateBillByOrderId(orderId);
            }

            try {
                codeUrl = WeixinPayCore.getCodeUrl(bill.getId(), "建筑服务安装费（"+orderId+"）", bill.getAmount(), NetUtil.getIpAddr(SpringContextUtil.getRequest()), "WIOS_EXPAND_PREDICT_FEE");
            } catch (Exception e) {
                logger.error("生成付款二维码出现异常:" + e.getMessage(), e);
                throw new ServiceException("生成付款二维码失败");
            }

            if(StringUtils.isBlank(codeUrl)){
                throw new ServiceException("生成付款二维码失败");
            }
            // 付款码2个小时内有效，本地缓存100分钟
//            redisClient.put(cacheKey, codeUrl, 6000);
        }
        return codeUrl;
    }

    private String getQrcodeCacheKey(String orderId){
        return PAY_QRCODE_CAHCE_KEY+orderId;
    }

    /**
     * 根据工单编号作废账单
     * 删除付款码缓存，并删除未支付的账单
     * @param orderId
     */
    public void deleteBill(String orderId){
        // 清理付款码缓存
        String cacheKey = getQrcodeCacheKey(orderId);
        redisClient.remove(cacheKey);

        UserPayBill bill = getByRefId(orderId);
        Assert.notNull(bill, "付款订单不存在");
        Assert.isTrue(bill.getStatus()==0, "不能作废，付款订单已支付");

        // 删除账单
        deleteById(bill.getId());
    }

    public int add(UserPayBill record){
        Assert.notBlank(record.getRefId(), "关联编号不能为空");

        String lockId="userPayBill_add_refId:"+record.getRefId();
        String requestId=StringUtilsEx.getUUID();
        if(redisClient.lock(lockId,requestId, 30)){
            try{
                if(StringUtils.isBlank(record.getId())){
                    record.setId(NumberUtil.getRandomOrderId());
                }
                if(record.getStatus() == null){
                    record.setStatus(0);
                }
                record.setCreateTime(new Date());
                if(tokenService.getUser()!=null){
                    record.setCreateAccount(tokenService.getUser().getUserId());
                }
                return userPayBillDao.add(record);
            }finally {
                redisClient.releaseLock(lockId, requestId);
            }
        }else{
            throw new ServiceException("系统忙，请稍后再试");
        }
    }

    public int update(UserPayBill record){
        Assert.notBlank(record.getId(), "ID不能为空");
        record.setModifyTime(new Date());
        if(tokenService.getUser()!=null){
            record.setModifyAccount(tokenService.getUser().getUserId());
        }
        return userPayBillDao.update(record);
    }

    /**
     * 付款成功
     * @param id
     * @param payType
     * @param payAccount
     * @param payTradeNo
     * @param payTime
     * @param source
     */
    public void paySuccess(String id, Integer payType, String payAccount, String payTradeNo, Date payTime, Integer source){
        UserPayBill bill = getById(id);
        Assert.notNull(bill, "账单不存在");
        paySuccess(bill, payType, payAccount, payTradeNo, payTime, source);
    }

    private void paySuccess(UserPayBill bill, Integer payType, String payAccount, String payTradeNo, Date payTime, Integer source){
        if(bill != null && (bill.getStatus()==null || bill.getStatus() == 0)){
            UserPayBill entity=new UserPayBill();
            entity.setId(bill.getId());
            entity.setPayType(payType);
            entity.setPayAccount(payAccount);
            entity.setPayTradeNo(payTradeNo);
            entity.setPayTime(payTime);
            entity.setSource(source);
            entity.setStatus(1);

            update(entity);

            OrderInstall orderInstall = new OrderInstall();
            orderInstall.setPayStatus(1);
            orderInstallService.updateByOrderId(bill.getRefId(), orderInstall);
        }
    }

    public void paySuccessByOrderId(String orderId, Integer payType, String payAccount, String payTradeNo, Date payTime, Integer source){
        UserPayBill bill = getByRefId(orderId);
        Assert.notNull(bill, "账单不存在");

        paySuccess(bill, payType, payAccount, payTradeNo, payTime, source);
    }

    public UserPayBill getById(String id){
        Assert.notBlank(id, "ID不能为空");
        return userPayBillDao.getById(id);
    }

    public int deleteById(String id){
        return userPayBillDao.deleteById(id);
    }

}
